import * as react from 'react';
import { UseQueryOptions, UseQueryResult } from '@tanstack/react-query';

type ServiceMethod<I, O> = (req: I, initReq: RequestInitWithPathPrefix) => Promise<O>;
interface RequestInitWithPathPrefix extends RequestInit {
    pathPrefix?: string;
}
type ServiceError = Error | ErrorResponse;
type OnErrorHandler<M extends ServiceMethod<I, O>, I, O> = (error: Error | ErrorResponse) => ReturnType<M> | null;
interface ErrorResponse {
    codeName?: string;
    code: number;
    message: string;
    details: unknown[];
}
type UseServiceQueryOptions<M extends ServiceMethod<I, O>, I, O> = Omit<UseQueryOptions<Awaited<O>, ServiceError>, 'queryFn'> & {
    onError?: OnErrorHandler<M, I, O>;
};
type UseServiceQueryResult<O> = UseQueryResult<Awaited<O>, ServiceError>;
declare function useServiceQuery<M extends ServiceMethod<I, O>, I, O>(method: M, req: I, options?: UseServiceQueryOptions<M, I, O>): UseServiceQueryResult<O>;
declare const ServiceContext: react.Context<RequestInitWithPathPrefix>;
declare function isErrorResponse(error: Error | ErrorResponse): error is ErrorResponse;

export { type ErrorResponse, type OnErrorHandler, type RequestInitWithPathPrefix, ServiceContext, type ServiceError, type ServiceMethod, isErrorResponse, useServiceQuery };
